/*
 * $Id: util.c,v 1.2 1998/03/18 16:18:58 labovit Exp $
 */

#include <sys/types.h> 
#include <sys/stat.h>
#include <fcntl.h>
#include <mrt.h>
#include <select.h>
#include "salserver.h"

extern trace_t			*default_trace; 
extern salserver_t		SALSERVER; 

void multiserver_connection_succeeded (sal_connection_t *multiserver);
void multiserver_connection_failed (sal_connection_t *multiserver);

int sal_add_channel (char *channel) {
  char *chan;

  LL_Iterate (SALSERVER.ll_channels, chan) {
    if (!strcmp (chan, channel)) {
      return (0);
    }
  }

  LL_Add (SALSERVER.ll_channels, strdup (channel));
  return (1);
}


/* sal_marshall_prop
 * Given a linked list of properties (key/value pairs), create
 * a properly NULL delinimited (yuck! evil ASCII protocol! yuck!)
 * Salamander property buffer
 */
int sal_marshall_prop (int query_id, LINKED_LIST *ll_prop, char *buf, int *n) {
  sal_property_t *prop;
  char *cp;

  cp = buf;
  LL_Iterate (ll_prop, prop) {
    memcpy (cp, prop->key, strlen (prop->key));
    cp += strlen (prop->key);
    *cp++ = '\0';

    memcpy (cp, prop->value, strlen (prop->value));
    cp += strlen (prop->value);
    *cp++ = '\0';
  }

  /* now add the query id */
  sprintf (cp, "ID"); 
  cp += strlen (cp);
  *cp++ = '\0';
  sprintf (cp, "%d", query_id); 
  cp += strlen (cp);
  *cp++ = '\0';


  *cp++ = '\0'; /* silly terminating null -- why? No need for this. Sigh. */
  *n = cp - buf;

  return (1);
}

/* marshalls linked list of strings -- not properties */
int sal_marshall (LINKED_LIST *ll_items, char *buf, int *n) {
  char *item;
  char *cp;

  cp = buf;
  LL_Iterate (ll_items, item) {
    memcpy (cp, item, strlen (item));
    cp += strlen (item);
    *cp++ = '\0';
  }

  *cp++ = '\0'; /* silly terminating null -- why? No need for this. Sigh. */
  *n = cp - buf;

  return (1);
}


/* sal_send_meta_response
 * We have been asked something about the channels we have available
 * This is a "meta-query". Send back a list of channels that match the
 * query 
 */
int sal_send_meta_response (sal_connection_t *connection, data_pdu_t *query) {
  data_pdu_t *data;
  sal_property_t *prop;
  char *buf, *cp, *channel;
  char tmp[10];
  
  buf = malloc (SAL_SMALL_BUF);

  cp = buf;
  memset (buf, 0, SAL_SMALL_BUF);
  LL_Iterate (SALSERVER.ll_channels,  channel) {

    if ((strcmp (query->name, "all") == 0) ||
	(strstr (channel, query->name) != NULL)) {
      sprintf (cp, "%s\n", channel);
      cp = cp +strlen (cp);
      /* too big */
      if ( (cp - buf) > SAL_SMALL_BUF - 1000) {
	sprintf (cp, "*** Truncated *****\n");
	cp = cp +strlen (cp);
	break;
      }
    } 
  }
  sprintf (tmp, "%d", cp - buf);

  data = New_Data_Pdu ();
  data->datalen = cp - buf;
  data->name = strdup ("Meta-Response");
  data->databuf = buf;

  prop = New (sal_property_t);
  prop->key = strdup ("OBJ_SIZE");
  prop->value = strdup (tmp);
  LL_Add (data->ll_properties, prop);

  prop = New (sal_property_t);
  prop->key = strdup ("COMMAND");
  prop->value = strdup ("META-QUERY RESPONSE");
  LL_Add (data->ll_properties, prop);

  sal_send_data (query->query_id, connection, data);

  Delete_Data (data);
  return (1);
}


/* sal_send_error
 * Send back an error message of string error 
 */
int sal_send_error (int query_id, sal_connection_t *connection, char *channel, char *error) {
  data_pdu_t *data;
  sal_property_t *prop;

  data = New_Data_Pdu ();
  data->datalen = 0;
  data->name = strdup ("Error");

  prop = New (sal_property_t);
  prop->key = strdup ("OBJ_SIZE");
  prop->value = strdup ("0");
  LL_Add (data->ll_properties, prop);

  prop = New (sal_property_t);
  prop->key = strdup ("SAL ERROR");
  prop->value = strdup (error);
  LL_Add (data->ll_properties, prop);

  prop = New (sal_property_t);
  prop->key = strdup ("COMMAND");
  prop->value = strdup ("QUERY RESPONSE");
  LL_Add (data->ll_properties, prop);

  sal_send_data (query_id, connection, data);
  Delete_Data (data);

  return (1);
}


int sal_change_property (LINKED_LIST *ll_properties, char *property, char *newvalue) {
  sal_property_t *prop = NULL;

  LL_Iterate (ll_properties, prop) {
    trace (TR_PACKET, default_trace, "  (Change) Searching for property %s/%s\n", 
	   property, prop->key);
    if (!strcmp (prop->key, property)) {
      Delete (prop->value);
      prop->value = strdup (newvalue);
      return (1);
    }
  }

  return (-1);
} 


char *sal_get_property (LINKED_LIST *ll_properties, char *property) {
  sal_property_t *prop = NULL;

  LL_Iterate (ll_properties, prop) {
    trace (TR_PACKET, default_trace, "  Searching for property %s/%s\n", 
	   property, prop->key);
    if (!strcmp (prop->key, property)) {
      trace (TR_PACKET, default_trace, "  Search successful! %s\n", 
	     prop->value);
      return (prop->value);
    }
  }

  trace (TR_PACKET, default_trace, "  Search failed\n");

  return (NULL);
} 


void Delete_Property (sal_property_t *p) {
  if (p == NULL) {return;}

  Delete (p->key);
  Delete (p->value);
  Delete (p);
}


  /* delete the data_pdu */
void Delete_Data (data_pdu_t *data) {
 
  LL_Destroy (data->ll_properties);
  Delete (data->name);
  if (data->propbuf != NULL) 
    Delete (data->propbuf);
  if (data->databuf != NULL) 
    Delete (data->databuf);
  Delete (data);

}


data_pdu_t *New_Data_Pdu () {
  data_pdu_t *tmp;

  tmp = New (data_pdu_t);
  tmp->ll_properties = LL_Create (LL_DestroyFunction, Delete_Property, NULL);
  tmp->cp = tmp->header;
  tmp->state = 0;
  tmp->propbuf = NULL;
  tmp->databuf = NULL;
  tmp->name = NULL;

  return (tmp);
}

sal_buffer_t *New_Sal_Buffer (int size) {
  sal_buffer_t *tmp;

  tmp = New (sal_buffer_t);
  tmp->data = malloc (size);
  tmp->cp = tmp->data;
  tmp->size = size;

  return (tmp);
}

void Delete_Sal_Buffer (sal_buffer_t *tmp) {
  Delete (tmp->data);
  Delete (tmp);
  return;
}
